
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include "interval.h"

bool interval_big (interval inter)
{
    return inter.up - inter.lo > 1;
}

bool interval_in (interval inter, int value)
{
    return inter.lo <= value && value < inter.up;
}

bool interval_intersect (interval inter1, interval inter2)
{
    return inter1.lo < inter2.up && inter2.lo < inter1.up;
}

bool interval_parse (const char * s, interval * intv,
                     char * err, size_t errsize)
{
    size_t p = 0;
    size_t deb;
    int n1, n2, sign, lo, up;
    bool bounds;
    if (s[p] != '(')
    {
        snprintf (err, errsize, "Expecting '(' at position %zu", p);
        return false;
    }
    p = 1;
    sign = +1;
    if (s[p] == '-' || s[p] == '+')
    {
        sign = s[p] == '-' ? -1 : +1;
        ++ p;
    }
    deb = p;
    while (isdigit (s[p]))
        ++ p;
    if (p == deb)
    {
        snprintf (err, errsize, "Expecting integer at position %zu", p);
        return false;
    }
    n1 = sign * atoi (s+deb);
    if (s[p] == ',')
        bounds = true;
    else if (s[p] == ':')
        bounds = false;
    else
    {
        snprintf (err, errsize, "Expecting ',' or ':' at position %zu", p);
        return false;
    }
    ++ p;
    sign = +1;
    if (bounds && (s[p] == '-' || s[p] == '+'))
    {
        sign = s[p] == '-' ? -1 : +1;
        ++ p;
    }
    deb = p;
    while (isdigit (s[p]))
        ++ p;
    if (p == deb)
    {
        snprintf (err, errsize, "Expecting integer at position %zu", p);
        return false;
    }
    n2 = sign * atoi (s+deb);
    if (s[p] != ')')
    {
        snprintf (err, errsize, "Expecting ')' at position %zu", p);
        return false;
    }
    p += 1;
    if (s[p] != '\0')
    {
        snprintf (err, errsize, "Excess characters at position %zu", p);
        return false;
    }
    lo = n1;
    up = bounds ? n2 : n1+n2;
    if (up < lo)
    {
        snprintf (err, errsize, "Interval (%d,%d) invalid", lo, up);
        return false;
    }
    intv->lo = n1;
    intv->up = bounds ? n2 : n1+n2;
    return true;

}
