
#include <stdbool.h> // these headers are needed for some of our code
#include <stdlib.h>

#include <check.h> // this is needed to use the "check" framework

#include "interval.h" // this is the library we want to test

// Below is a first collection of unit tests for the big function.
//
// Every test has to be written as:
//
// START_TEST (<test_name>)
// {
//     <test_code>
// }
//
// where <test_name> is a valid name for a function (or variable)
// and <test_code> is the C code of our unit test. The code must be
// kept as simple as possible (we will *not* write tests for the tests).
//
// Every unit test ends with an instruction like "ck_assert (...)"
// where the parameter is a boolean expression. The tests succeeds
// if the boolean expression evaluates to true, fails otherwise.
// You can loop up the documentation of the "check" framework for
// variants of ck_assert ().
//
// Note that each unit tests can create its own local variables.
// You can also use global variables if needed.

//
// big
//

START_TEST (test_big_0)
{
    interval intv = { .lo=0, .up=0 };
    ck_assert (interval_big (intv) == false);
}
END_TEST

START_TEST (test_big_1)
{
    interval intv = { .lo=0, .up=1 };
    ck_assert (interval_big (intv) == false);
}
END_TEST

START_TEST (test_big_2)
{
    interval intv = { .lo=0, .up=2 };
    ck_assert (interval_big (intv) == true);
}
END_TEST

// Here you add your own unit tests, and then you need to update
// the suite() function below.

//
// in
//

/*
START_TEST (test_in_0)
{
    interval intv = { .lo=0, .up= 0};
    ck_assert (interval_in(intv,0) == false);
}
END_TEST

START_TEST (test_in_1)
{
    interval intv = { .lo=0, .up= 1};
    ck_assert (interval_in(intv,0) == true);
}
END_TEST

START_TEST (test_in_2)
{
    interval intv = { .lo=0, .up= 1};
    ck_assert (interval_in(intv,1) == false);
}
END_TEST
*/

// Tableaux des bornes de l'intervalle, des valeurs à tester, et des résultats attendus
int lo_values[] = { 0, 0, 0, 5, 5, 10 };
int up_values[] = { 0, 1, 1, 10, 10, 20 };
int test_values[] = { 0, 0, 1, 7, 10, 15 };
bool expected_results[] = { false, true, false, true, false, true };

// Fonction pour tester tous les cas de test pour interval_in
START_TEST(test_in_loop)
{
    size_t i;
    for (i = 0; i < sizeof(lo_values) / sizeof(lo_values[0]); i++) 
    {
        interval intv = { .lo = lo_values[i], .up = up_values[i] };
        bool result = interval_in(intv, test_values[i]);
        ck_assert_int_eq(result, expected_results[i]);
    }
}
END_TEST



//
// setting up a suite
//

// This function creates a test suite. The structure is the following:
//
// - a test suite (type Suite) can contain several test cases
// - a test case (type TCase) can contain several unit test
// - a unit test is create with START_TEST/END_TEST (as above)
//
// All elements of the test suite must be dynamically allocated, with
// suite_create()/tcase_create(). Every test case must be added to the
// suite with suite_add_tcase(). Every test must be added to a test
// case with tcase_add_test().
//
// The following function does all this. It needs to be modified every
// time you create a new test case or unit test.

Suite * suite(void)
{
    Suite * s = suite_create ("Interval");
    TCase * tc_big = tcase_create ("big");
    suite_add_tcase (s, tc_big);
    {
        tcase_add_test (tc_big, test_big_0);
        tcase_add_test (tc_big, test_big_1);
        tcase_add_test (tc_big, test_big_2);
    }

    TCase * tc_in = tcase_create("in");
    suite_add_tcase (s,tc_in);
    /*
    {
        tcase_add_test (tc_in, test_in_0);
        tcase_add_test (tc_in, test_in_1);
        tcase_add_test (tc_in, test_in_2);
    }
    */
    tcase_add_test(tc_in, test_in_loop);

    return s;
}

// The main function creates the test suite (via suite() above),
// and then creates a "runner" (type SRunner) that will control
// the execution of all unit tests.
//
// Then the runner run all tests, via srunner_run_all().
//
// Then you can retrieve some statistics, free all allocated memory
// with a single call to srunner_free(), and exit with an
// appropriate code.

int main (void)
{
    int number_failed;
    Suite * s = suite();
    SRunner * sr = srunner_create(s);

    srunner_run_all(sr, /*CK_NORMAL*/ CK_VERBOSE);

    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);
    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
